// _page.scss
// basic page definitions

html,
body {
		font-family: "HelveticaNeue-Light", "Helvetica Neue Light", "Helvetica Neue", Helvetica, Arial, "Lucida Grande", sans-serif;
		font-weight: 300;
}

// container widths
.container{
		@media (min-width: 992px) {
				width: 97%;
				max-width: 1100px;
		}

		@media (min-width: 1200px) {
				width: 97%;
				max-width: 1400px;
		}
}
