/*
 * Copyright (C) Alexander Urban (2018)
 *
 * This file is part of GWDetChar
 *
 * GWDetChar is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GWDetChar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GWDetChar.  If not, see <http://www.gnu.org/licenses/>
 */

body {
		font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
		font-weight: 300;
}

h1, h2, h3, h4, h5, h6 {
		font-weight: 300;
}

h4 {
		word-break: break-all;
}

h2::before, h3::before, h4::before, h5::before, h6::before {
		display: block;
		content: "";
		margin-top: -70px;
		height: 70px;
		visibility: hidden;
		pointer-events: none;
}

a {
		cursor: pointer;
}

img {
		margin: 0 auto;
}

.navbar {
		.ifo-links  {
				.dropdown-header {
						padding-left: 20px;
						padding-right: 20px;
						white-space: normal;;
				}
				@media (min-width: 768px) {
						position: absolute;
						right: 0;
				}
		}
}

.dropdown-1-col {
		white-space: nowrap;
		max-height: 700px;
		overflow-y: scroll;
}

.container {
		width: 97%;
		@media (max-width: 991px) {
				max-width: 991px;
		}
		@media (min-width: 992px) {
				max-width: 1100px;
		}

		@media (min-width: 1200px) {
				max-width: 1400px;
		}
}

.btn {
		opacity: 0.8;
}

.btn-group {
		margin-top: 0px;
		margin-bottom: 6px;
		@media(max-width:991px) {
				float: left;
		}
		@media(min-width:992px) {
				float: right;
		}
}

.btn-table {
		margin-top: -15px;
		margin-bottom: 15px;
}

.desktop-only {
		@media(max-width:991px) {
				display:none;
		}
}
