// Bootstrap CSS extensions for LIGO HTML

// variables and functions
@import 'variables';
@import 'mixins';

// basic styling
@import 'page';

// navbar and header components
@import 'navbar';
@import 'calendar';
@import 'banner';

// main body components
@import 'scaffolding';
@import 'code';
@import 'panels';
@import 'images';

// footer
@import 'footer';

// extras
@import 'ifos';
@import 'fancybox';
