// _mixins.scss

@mixin no-text-select() {
		-webkit-user-select: none;
		-moz-user-select: none;
		-ms-user-select: none;
		user-select: none;
}

@mixin background-panel($color, $height) {
		background-image: none;
		background-image: -ms-linear-gradient(bottom, $color 0%, $color 100%);
		background-image: -moz-linear-gradient(bottom, $color 0%, $color 100%);
		background-image: -o-linear-gradient(bottom, $color 0%, $color 100%);
		background-image: -webkit-gradient(linear, left bottom, left top, color-stop(0, $color), color-stop(1, $color));
		background-image: -webkit-linear-gradient(bottom, $color 0%, $color 100%);
		background-image: linear-gradient(to top, $color 0%, $color 100%);
		background-size: 100% $height;
		background-repeat: no-repeat;
		@media (min-width: $navbar-breakpoint) {
				background-color: $color;
		}
}
