// _fancybox.scss
// styling for fancyboxes

.fancybox {
		max-width: 100%;
}

.fancybox-skin {
		background: white;
}

.fancybox-nav {
		height: 10%;
		width: 5%;
}

.fancybox-title {
		text-align: center;
		font-size: 16px;
}
