// _footer.scss

$footer-height: 140px;
$footer-padding: 20px;

html {
		position: relative;
		min-height: 100%;
}

body {
		margin-bottom: $footer-height + $footer-padding * 2;
}

.footer {
		position: absolute;
		bottom: 0;
		width:100%;
		height: $footer-height;
		padding-top: $footer-padding;
		padding-bottom: $footer-padding;
}

footer {
		clear: both;
		font-size: small;
		border-top: 1px solid gray;
		background-color: #555;

		&, a {
				color: #eee;
		}

		> .container {
				margin-top: 20px;
				margin-bottom: 20px;
		}

		p, a {
				padding: 2px 0px;
				margin: 0;
		}
		p {
				display: block;
		}

		a:hover {
				text-decoration: underline;
				color: white;
		}
}
